@@ -3,78 +3,102 @@ use std::str::FromStr;
 use std::cmp;
 use std::sync::Arc;
 
-use num::{BigRational, FromPrimitive};
+use num::{BigInt, BigRational, FromPrimitive, One, Zero, ToPrimitive};
 
 #[cfg(feature = "compile")]
 use serde::{Serialize, Deserialize};
 
-use crate::{Env, Error, Exprs, FnArgs, FnReturn, Lines, Op, OpTrait, Pos, TVal, Token, TokenType, Tokens, Type, Value, VecMap, umcore, unescape};
+use crate::{Env, Error, Exprs, FnArgs, FnReturn, Lines, Op, OpTrait, Pos, TVal, RefTVal, Token, TokenType, Tokens, Type, Value, VecMap, umcore, unescape};
 
+/// The parsed representation of a script expression.
 #[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
 #[cfg_attr(feature = "compile", derive(Serialize, Deserialize))]
 pub enum AST {
+    /// A function call that accepts tokens as arguments.
     Macro {
         tokens: Vec<Token>,
     },
+    /// A collection of other expressions.
     Container {
         token: Token,
         children: Vec<AST>,
     },
+    /// An operation between two operands.
     Operator {
         token: Token,
         lhs: Box<AST>,
         rhs: Box<AST>,
     },
+    /// A singular value.
     Value {
         token: Token,
     },
+    /// The empty AST, used to represent an empty child. Cannot be run.
     Empty,
 }
 impl AST {
-    pub fn from_value(v: &Value) -> Option<Self> {
+    pub(crate) fn from_value(v: &Value) -> Option<Self> {
         if let Value::Enum(_es, e) = v {
             if e.0 == "Empty" {
                 return Some(AST::Empty);
             }
 
-            let st = match &e.1 {
-                Some(TVal { val: Value::Struct(st), .. }) => st,
+            let st = match &e.1.get(0) {
+                Some(rv) => {
+                    if let TVal { val: Value::Struct(st), .. } = rv.clone_out() {
+                        st
+                    } else {
+                        return None
+                    }
+                },
                 _ => return None,
             };
             match &*e.0 {
                 "Macro" => return Some(AST::Macro {
                     tokens: match st.get("tokens") {
-                        Some(TVal { val: Value::List(l), .. }) => l.iter().map(|tv| Token::from_value(&tv.val)).collect::<Option<Vec<Token>>>()?,
+                        Some(rv) => {
+                            if let TVal { val: Value::List(l), .. } = rv.clone_out() {
+                                l.iter().map(|tv| Token::from_value(&tv.clone_out().val)).collect::<Option<Vec<Token>>>()?
+                            } else {
+                                return None
+                            }
+                        },
                         _ => return None,
                     },
                 }),
                 "Container" => return Some(AST::Container {
                     token: match st.get("token") {
-                        Some(tv) => Token::from_value(&tv.val)?,
+                        Some(tv) => Token::from_value(&tv.clone_out().val)?,
                         _ => return None,
                     },
                     children: match st.get("children") {
-                        Some(TVal { val: Value::List(l), ..}) => l.iter().map(|tv| AST::from_value(&tv.val)).collect::<Option<Vec<AST>>>()?,
+                        Some(rv) => {
+                            if let TVal { val: Value::List(l), ..} = rv.clone_out() {
+                                l.iter().map(|tv| AST::from_value(&tv.clone_out().val)).collect::<Option<Vec<AST>>>()?
+                            } else {
+                                return None
+                            }
+                        },
                         _ => return None,
                     },
                 }),
                 "Operator" => return Some(AST::Operator {
                     token: match st.get("token") {
-                        Some(tv) => Token::from_value(&tv.val)?,
+                        Some(tv) => Token::from_value(&tv.clone_out().val)?,
                         _ => return None,
                     },
                     lhs: match st.get("lhs") {
-                        Some(tv) => Box::new(AST::from_value(&tv.val)?),
+                        Some(tv) => Box::new(AST::from_value(&tv.clone_out().val)?),
                         _ => return None,
                     },
                     rhs: match st.get("rhs") {
-                        Some(tv) => Box::new(AST::from_value(&tv.val)?),
+                        Some(tv) => Box::new(AST::from_value(&tv.clone_out().val)?),
                         _ => return None,
                     },
                 }),
                 "Value" => return Some(AST::Value {
                     token: match st.get("token") {
-                        Some(tv) => Token::from_value(&tv.val)?,
+                        Some(tv) => Token::from_value(&tv.clone_out().val)?,
                         _ => return None,
                     },
                 }),
@@ -84,25 +108,28 @@ impl AST {
         None
     }
 
-    pub fn get_first_token(&self) -> Option<Token> {
-        use AST::*;
+    pub(crate) fn get_first_token(&self) -> Option<Token> {
         match self {
-            Macro { tokens } => tokens.first().cloned(),
-            Container { token, .. } => Some(token.clone()),
-            Operator { token, lhs, .. } => match **lhs {
-                Empty => Some(token.clone()),
+            AST::Macro { tokens } => tokens.first().cloned(),
+            AST::Container { token, .. } | AST::Value { token } => Some(token.clone()),
+            AST::Operator { token, lhs, .. } => match **lhs {
+                AST::Empty => Some(token.clone()),
                 _ => lhs.get_first_token(),
             },
-            Value { token } => Some(token.clone()),
-            Empty => None,
+            AST::Empty => None,
         }
     }
-    pub fn get_first_pos(&self) -> Option<Pos> {
+    pub(crate) fn get_first_pos(&self) -> Option<Pos> {
         match self.get_first_token() {
             Some(t) => Some(t.pos),
             None => None,
         }
     }
+    /**
+    Returns a `String` representing the `AST`. Not guaranteed to be the same
+    as the source script.
+    */
+    #[must_use]
     pub fn to_string_lossy(&self) -> String {
         match self {
             AST::Macro { tokens } => tokens.iter()
@@ -112,20 +139,20 @@ impl AST {
             AST::Container { token, children } => match &*token.data {
                 "(" | "[" => {
                     let cs = children.iter()
-                        .map(|a| a.to_string_lossy())
+                        .map(AST::to_string_lossy)
                         .collect::<Vec<String>>()
                         .join(", ");
                     format!("{}{}{}", token.data, cs, if token.data == "(" { ")" } else { "]" })
                 },
                 "{" => {
-                    if children.len() > 1 {
+                    if children.is_empty() {
+                        "{}".into()
+                    } else {
                         let cs = children.iter()
-                            .map(|a| a.to_string_lossy())
+                            .map(AST::to_string_lossy)
                             .collect::<Vec<String>>()
                             .join(";\n\t");
                         format!("{}\n\t{};\n{}", token.data, cs, "}")
-                    } else {
-                        "{}".into()
                     }
                 },
                 s if s.starts_with("//") => token.data.clone(),
@@ -140,6 +167,13 @@ impl AST {
         }
     }
 
+    /// Parses the `Token` stream into a single `AST` representing the entire
+    /// expression.
+    ///
+    /// # Errors
+    /// Will return `Err` if `parse()` on children resulted in an error.
+    /// An internal function will error if it's unable to reduce adjacent ASTs
+    /// into a single AST.
     pub fn parse<'a, I>(tokens: I, env: &Env<'a>) -> Result<AST, Error>
     where
         I: IntoIterator<Item = Token>
@@ -152,25 +186,26 @@ impl AST {
                     return acc;
                 }
 
-                use TokenType::*;
                 match t.ttype {
-                    Identifier => {
-                        if let Some(TVal { val: Value::Function { args, .. }, .. }) = env.get(&t.data) {
-                            if args == &umcore::_macro_args()
-                                || (args.contains_key("env") && args.contains_key("pos") && args.contains_key("tokens"))
-                            {
-                                acc.push(Ok(AST::Macro { tokens: tokens[i..].to_vec() }));
-                                return acc;
+                    TokenType::Identifier => {
+                        if let Some(rv) = env.get(&t.data) {
+                            if let TVal { val: Value::Function { args, .. }, .. } = rv.clone_out() {
+                                if args == umcore::_macro_args()
+                                    || (args.contains_key("env") && args.contains_key("pos") && args.contains_key("tokens"))
+                                {
+                                    acc.push(Ok(AST::Macro { tokens: tokens[i..].to_vec() }));
+                                    return acc;
+                                }
                             }
                         }
                         acc.push(Ok(AST::Value { token: t.clone() }));
                         acc
                     },
-                    Number | String => {
+                    TokenType::Number | TokenType::String => {
                         acc.push(Ok(AST::Value { token: t.clone() }));
                         acc
                     },
-                    Container => {
+                    TokenType::Container => {
                         let children = Lines::split(&t.pos.filename, &t.data[1..cmp::max(1, t.data.len()-1)], Some(Pos {
                                 filename: t.pos.filename.clone(),
                                 line: t.pos.line,
@@ -182,7 +217,7 @@ impl AST {
                         match children {
                             Ok(cs) => acc.push(Ok(AST::Container {
                                 token: Token {
-                                    ttype: Container,
+                                    ttype: TokenType::Container,
                                     pos: t.pos.clone(),
                                     data: std::string::String::from(t.data.as_bytes()[0] as char),
                                 },
@@ -192,7 +227,7 @@ impl AST {
                         }
                         acc
                     },
-                    Symbol => {
+                    TokenType::Symbol => {
                         acc.push(Ok(AST::Operator {
                             token: t.clone(),
                             lhs: Box::new(AST::Empty),
@@ -200,7 +235,7 @@ impl AST {
                         }));
                         acc
                     },
-                    Comment => {
+                    TokenType::Comment => {
                         acc.push(Ok(AST::Container {
                             token: t.clone(),
                             children: vec![],
@@ -214,11 +249,9 @@ impl AST {
                 return Err(m.clone());
             }
         }
-        let asts = asts.drain(0..)
-            .map(|a| a.unwrap())
-            .collect();
+        let asts: Result<Vec<AST>, Error> = asts.drain(0..).collect();
 
-        AST::group_precedence(asts)
+        AST::group_precedence(asts?)
     }
     fn group_precedence(mut asts: Vec<AST>) -> Result<AST, Error> {
         // TODO actually determine precedence
@@ -227,7 +260,7 @@ impl AST {
             let mut unchanged = 0;
             while i < asts.len() {
                 match asts[i].clone() {
-                    AST::Operator { token, lhs, rhs } if token.data == "." && *lhs == AST::Empty && *rhs == AST::Empty => {
+                    AST::Operator { token, lhs, rhs } if (token.data == "." || token.data == "@") && *lhs == AST::Empty && *rhs == AST::Empty => {
                         let lhs = if i > 0 {
                             asts.remove(i-1)
                         } else {
@@ -255,7 +288,9 @@ impl AST {
                     },
                     AST::Container { token, children } if i > 0 => {
                         match &asts[i-1] {
-                            AST::Operator { rhs, .. } if **rhs == AST::Empty =>  {},
+                            AST::Operator { rhs, .. } if **rhs == AST::Empty =>  {
+                                unchanged = 0;
+                            },
                             _ => {
                                 let lhs = asts.remove(i-1);
                                 let _cont = asts.remove(i-1); // index changed from previous remove
@@ -329,21 +364,15 @@ impl AST {
                         });
                     },
 
-                    // TODO handle other cases as needed
-                    // AST::Macro { .. } => {},
-                    // AST::Container { .. } => {},
-                    // AST::Operator { .. } => {},
-                    AST::Value { .. } => {
+                    AST::Macro { .. } | AST::Container { .. } | AST::Value { .. } => {
                         unchanged = 0;
                     },
-                    // AST::Empty { .. } => {},
                     _ => {
-                        // println!("{:?}", asts[i]);
+                        // eprintln!("{:?}", asts[i]);
                     },
                 }
-                if unchanged > 1 {
-                    // TODO fix fib.um
-                    return Err(Error::ParseError(
+                if unchanged > 0 {
+                    return Err(Error::Parse(
                         format!("Unreducible values [{}/{}]: {:?}", i+1, asts.len(), asts.iter()
                             .map(|a| "<".to_string()+&a.to_string_lossy()+">")
                             .intersperse(" ".into())
@@ -355,42 +384,59 @@ impl AST {
                 i += 1;
             }
         }
-        Ok(asts.remove(0))
+        if asts.is_empty() {
+            Ok(AST::Empty)
+        } else {
+            Ok(asts.remove(0))
+        }
     }
 
+    /**
+    Runs the `AST` in the given environment and returns the value of the
+    expression.
+    */
     pub fn run<'a>(&self, env: &'a Env<'a>) -> FnReturn {
         match self {
             AST::Macro { tokens } => {
                 let t0 = tokens.first().expect("expected macro token");
-                if let Some(TVal { val: Value::Function { body, .. }, .. }) = env.get(&t0.data) {
-                    return body.call(&*env, FnArgs::Macro {
-                        vars: Env::flatten(env).vars.into(),
-                        pos: self.get_first_pos(),
-                        tokens: tokens.clone(),
-                    });
+                if let Some(rv) = env.get(&t0.data) {
+                    if let TVal { val: Value::Function { body, .. }, .. } = rv.clone_out() {
+                        return body.call(&*env, FnArgs::Macro {
+                            vars: Env::flatten(env).vars.into(),
+                            pos: self.get_first_pos(),
+                            tokens: tokens.clone(),
+                        });
+                    }
                 }
-                return (None, Err(Error::ScriptError(format!("failed to find macro {} in env", t0.data), self.get_first_pos())));
+                return (None, Err(Error::Script(format!("failed to find macro {} in env", t0.data), self.get_first_pos())));
             },
             AST::Container { token, children } => {
                 let mut nenv = Env::child(&env);
                 let mut should_map = false;
-                let vals: Result<Vec<TVal>, Error> = children.iter()
-                    .map(|ast| {
+                let vals: Result<Vec<RefTVal>, Error> = children.iter()
+                    .filter_map(|ast| {
                         if let AST::Operator { token, .. } = &ast {
                             if token.data == ":" {
                                 should_map = true;
                             }
+                        } else if let AST::Container { token, .. } = &ast {
+                            // Skip comments
+                            if token.data.starts_with("//") {
+                                return None;
+                            }
                         }
 
                         // Run child AST
                         let (vars, v) = ast.run(&nenv);
                         if let Some(vars) = vars {
                             nenv.update(vars);
+                            // eprintln!("{:?}", vars);
+                            // crate::run_interactive(&nenv);
                         }
-                        v
+                        Some(v)
                     }).collect();
-                let diff = env.diff(&mut nenv);
-                let vals: Vec<TVal> = match vals {
+                let diff = env.diff(&mut nenv.clone());
+                let vals: Vec<RefTVal> = match vals {
                     Ok(vs) => vs,
                     Err(m) => return (Some(diff), Err(m)),
                 };
@@ -403,31 +449,27 @@ impl AST {
                         }
                         return (Some(diff), Ok(vals.last().unwrap_or(&Value::none().into()).clone()))
                     },
-                    "[" => return (Some(diff), Ok(Value::List(vals).into())),
+                    "[" => return (Some(nenv.vars), Ok(Value::List(vals).into())),
                     "{" => {
                         // Attempt to build Map from kv pairs
                         if should_map {
-                            let map: Result<Vec<(TVal, TVal)>, Error> = vals.iter()
-                                .map(|tv| {
-                                    if let TVal { val: Value::List(l), .. } = tv {
+                            let map: Result<Vec<(RefTVal, RefTVal)>, Error> = vals.iter()
+                                .map(|rv| {
+                                    if let TVal { val: Value::List(l), .. } = rv.clone_out() {
                                         if l.len() == 2 {
                                             return Ok((l[0].clone(), l[1].clone()));
                                         }
                                     }
-                                    return Err(Error::ScriptError(format!("failed to build Map from Container{{}}: {} is not a List", tv), Some(token.pos.clone())));
+                                    return Err(Error::Script(format!("failed to build Map from Container{{}}: {} is not a List", rv), Some(token.pos.clone())));
                                 }).collect();
                             if let Ok(m) = map {
-                                return (Some(diff), Ok(Value::Map(m.into_iter().collect::<HashMap<TVal, TVal>>().into()).into()));
+                                return (Some(diff), Ok(Value::Map(m.into_iter().collect::<HashMap<RefTVal, RefTVal>>().into()).into()));
                             }
                         }
                         return (Some(diff), Ok(vals.last().unwrap_or(&Value::none().into()).clone()));
                     },
                     _ => {
-                        // Ignore comments
-                        if token.data.starts_with("//") {
-                            return (None, Ok(Value::none().into()));
-                        }
-                        return (None, Err(Error::ScriptError(format!("unsupported Container {}", token.data), self.get_first_pos())));
+                        return (None, Err(Error::Script(format!("unsupported Container {}", token.data), self.get_first_pos())));
                     },
                 }
             },
@@ -435,11 +477,23 @@ impl AST {
                 let mut lval = None;
                 let mut rval = None;
 
-                // Handle the Equal operator without evaluation the LHS
+                // Handle the Equal operator without evaluating the LHS
                 if token.data == "=" {
-                    let lval: TVal = match &**lhs {
+                    let lval: RefTVal = match &**lhs {
                         AST::Value { token: lt } => Value::String(lt.data.clone()).into(),
-                        _ => return (None, Err(Error::ScriptError(format!("invalid LHS for Equal: {:?}", lhs), Some(token.pos.clone())))),
+                        AST::Operator { token: op, .. } if op.data == "." || op.data == "@" => {
+                            match lhs.run(env).1 {
+                                Ok(v) => v,
+                                Err(m) => return (None, Err(m)),
+                            }
+                        },
+                        AST::Operator { token: op, lhs: llhs, rhs: lrhs } if op.data == "*" && **llhs == AST::Empty => {
+                            match lrhs.run(env).1 {
+                                Ok(v) => v,
+                                Err(m) => return (None, Err(m)),
+                            }
+                        },
+                        _ => return (None, Err(Error::Script(format!("invalid LHS for Equal: {:?}", lhs), Some(token.pos.clone())))),
                     };
                     let rval = match rhs.run(env).1 {
                         Ok(v) => Some(v),
@@ -456,25 +510,42 @@ impl AST {
                     };
                 }
 
-                // Handle Dot operator without evaluating the RHS
-                if token.data == "." {
-                    let rstr = rhs.get_first_token().expect("expected RHS token").data;
-                    let rval = Some(Value::String(rstr.clone()).into());
+                // Handle Dot and Ampersat operators without evaluating the RHS
+                if token.data == "." || token.data == "@" {
+                    let (rstr, rval): (String, Option<RefTVal>) = match &**rhs {
+                        AST::Value { token: rtoken } => {
+                            let rstr = rtoken.data.clone();
+                            (rstr.clone(), Some(Value::String(rstr).into()))
+                        },
+                        AST::Container { .. } => {
+                            // Only evaluate RHS if it's in a container
+                            match rhs.run(env) {
+                                (_, Ok(v)) => ("".into(), Some(v)),
+                                (vars, Err(m)) => return (vars, Err(m)),
+                            }
+                        },
+                        _ => {
+                            let rstr = rhs.get_first_token().expect("expected RHS token").data;
+                            (rstr.clone(), Some(Value::String(rstr).into()))
+                        },
+                    };
                     match lval {
-                        Some(lval) => return OpTrait::<'a, {Op::Dot}>::op(&lval, &env, token.pos.clone(), rval.as_ref()),
-                        None => {
+                        Some(lval) => return Op::op(&env, token.clone(), Some(&lval), rval.as_ref()),
+                        None if token.data == "." => {
                             // If no LHS then fetch from the Enums in the current env
                             for (_pek, pev) in Env::iter_flatten(&env) {
-                                if let TVal { val: Value::Type(en, enr), .. } = pev {
-                                    if let Type::Enum(envar) = &**en {
+                                if let TVal { val: Value::Type(en, enr), .. } = pev.clone_out() {
+                                    if let Type::Enum(envar) = &*en {
                                         if enr.get(&rstr).is_some() {
-                                            let v = Value::Enum(Arc::clone(envar), Box::new((rstr, None)));
+                                            let v = Value::Enum(Arc::clone(envar), Box::new((rstr, vec![])));
                                             return (None, Ok(v.into()));
                                         }
                                     }
                                 }
                             }
                         },
+                        None if token.data == "@" => {},
+                        None => {},
                     }
                 }
 
@@ -495,29 +566,48 @@ impl AST {
                 return (nvars, val);
             },
             AST::Value { token } => {
-                use TokenType::*;
                 match token.ttype {
-                    Identifier => match env.get(&token.data) {
+                    TokenType::Identifier => match env.get(&token.data) {
                         Some(v) => return (None, Ok(v.clone())),
-                        None => return (None, Err(Error::ScriptError(format!("failed to find '{}' in this env", token.data), self.get_first_pos()))),
+                        None => return (None, Err(Error::Script(format!("failed to find '{}' in this env", token.data), self.get_first_pos()))),
                     },
-                    Number => {
+                    TokenType::Number => {
                         match BigRational::from_str(&token.data) {
                             Ok(r) => return (None, Ok(Value::Number(r).into())),
-                            Err(_) => if let Ok(f) = f64::from_str(&token.data) {
-                                if let Some(r) = BigRational::from_f64(f) {
-                                    return (None, Ok(Value::Number(r).into()));
+                            Err(_m) => {
+                                let (mut td, ordmag) = if let Some(eidx) = token.data.to_lowercase().find('e') {
+                                    (token.data[..eidx].to_string(), match BigInt::from_str(&token.data[eidx+1..]) {
+                                        Ok(v) => v,
+                                        Err(m) => return (None, Err(Error::Script(format!("failed to parse '{}' as number: {}", token.data, m), self.get_first_pos()))),
+                                    })
+                                } else {
+                                    (token.data.clone(), BigInt::zero())
+                                };
+
+                                let denom = if let Some(didx) = td.find('.') {
+                                    td.remove(didx);
+                                    BigInt::from_usize(td.len() - didx).unwrap_or_else(BigInt::one)
+                                } else {
+                                    BigInt::zero()
+                                };
+
+                                match (|| -> Option<BigRational> {
+                                    let ten = BigRational::from_i64(10)?;
+                                    let r = BigRational::from_str(&td).ok()?;
+                                    Some(r * ten.pow((ordmag -denom).to_i32()?))
+                                })() {
+                                    Some(r) => return (None, Ok(Value::Number(r).into())),
+                                    None => return (None, Err(Error::Script(format!("failed to parse '{}' as number", token.data), self.get_first_pos()))),
                                 }
                             },
                         }
-                        return (None, Err(Error::ScriptError(format!("failed to parse '{}' as number", token.data), self.get_first_pos())));
                     },
-                    String => return (None, Ok(Value::String(unescape(&token.data[1..token.data.len()-1])).into())),
+                    TokenType::String => return (None, Ok(Value::String(unescape(&token.data[1..token.data.len()-1])).into())),
                     _ => {},
                 }
             },
             _ => {},
         }
-        (None, Err(Error::ScriptError(format!("failed to execute AST: {:?}", self), self.get_first_pos())))
+        (None, Err(Error::Script(format!("failed to execute AST: {:?}", self), self.get_first_pos())))
     }
 }
