@@ -7,6 +7,7 @@ use serde::{Serialize, Deserialize};
 
 use crate::Value;
 
+/// Denotes a specific position within a script.
 #[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
 #[cfg_attr(feature = "compile", derive(Serialize, Deserialize))]
 pub struct Pos {
@@ -15,6 +16,11 @@ pub struct Pos {
     pub col: usize,
 }
 impl Pos {
+    /**
+    Constructs a `Pos` at the beginning of the given filename, i.e. line 1
+    column 1.
+    */
+    #[must_use]
     pub fn start(filename: &str) -> Self {
         Pos {
             filename: filename.into(),
@@ -22,18 +28,18 @@ impl Pos {
             col: 1,
         }
     }
-    pub fn from_value(v: &Value) -> Option<Self> {
+    pub(crate) fn from_value(v: &Value) -> Option<Self> {
         if let Value::Struct(hm) = v {
             return Some(Pos {
-                filename: match &hm.get("filename")?.val {
+                filename: match &hm.get("filename")?.clone_out().val {
                     Value::String(s) => s.clone(),
                     _ => return None,
                 },
-                line: match &hm.get("line")?.val {
+                line: match &hm.get("line")?.clone_out().val {
                     Value::Number(n) => n.to_usize()?,
                     _ => return None,
                 },
-                col: match &hm.get("col")?.val {
+                col: match &hm.get("col")?.clone_out().val {
                     Value::Number(n) => n.to_usize()?,
                     _ => return  None,
                 },
@@ -83,7 +89,12 @@ impl<'a> SplitItem<'a> {
         };
         match amount {
             SplitAmount::Set(a) => line = a,
-            SplitAmount::Increase(a) => line = (line as isize + a) as usize,
+            SplitAmount::Increase(a) => {
+                let iline = line.to_isize().unwrap();
+                if a <= iline {
+                    line = (iline + a).to_usize().unwrap();
+                }
+            },
         };
         match self {
             SplitItem::Lines(ls) => ls.pos.line = line,
@@ -97,7 +108,12 @@ impl<'a> SplitItem<'a> {
         };
         match amount {
             SplitAmount::Set(a) => col = a,
-            SplitAmount::Increase(a) => col = (col as isize + a) as usize,
+            SplitAmount::Increase(a) => {
+                let icol = col.to_isize().unwrap();
+                if a <= icol {
+                    col = (icol + a).to_usize().unwrap();
+                }
+            },
         }
         match self {
             SplitItem::Lines(ls) => ls.pos.col = col,
@@ -121,7 +137,7 @@ where
     let mut sc: usize = 0;
     // Eat whitespace before token
     while sc < remainder.len() && remainder[sc].is_whitespace() {
-        match remainder[sc] {
+        match remainder.get(sc).unwrap() {
             '\n' => {
                 si.pos_set_line(SplitAmount::Increase(1));
                 si.pos_set_col(SplitAmount::Set(1));
@@ -137,8 +153,8 @@ where
     let mut containers: Vec<char> = vec![];
     let mut prev: Option<char> = None;
     while sc < remainder.len() {
-        match remainder[sc] {
-            c if c == schar => {
+        match remainder.get(sc).unwrap() {
+            c if c == &schar => {
                 if containers.is_empty() {
                     break;
                 }
@@ -272,33 +288,50 @@ impl Iterator for Exprs {
     }
 }
 
+/// Easily differentiates `Token`s.
 #[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
 #[cfg_attr(feature = "compile", derive(Serialize, Deserialize))]
 pub enum TokenType {
+    /// Anything alphanumeric not beginning with a number.
     Identifier,
+    /// Anything numeric containing at most one `.` and `e` each.
     Number,
+    /**
+    Anything enclosed by double quotes.
+    May contain escaped quotes, newlines, tabs, and backslashes.
+    */
     String,
+    /**
+    A collection of untokenized data.
+    Enclosed in either `()`, `[]`, or `{}`.
+    */
     Container,
+    /// Another symbol, typically an operator. May have multiple characters.
     Symbol,
     Comment,
 }
 impl TokenType {
     fn from_value(v: &Value) -> Option<Self> {
         if let Value::Enum(_es, e) = v {
-            use TokenType::*;
             return Some(match &*(**e).0 {
-                "Identifier" => Identifier,
-                "Number" => Number,
-                "String" => String,
-                "Container" => Container,
-                "Symbol" => Symbol,
-                "Comment" => Comment,
+                "Identifier" => TokenType::Identifier,
+                "Number" => TokenType::Number,
+                "String" => TokenType::String,
+                "Container" => TokenType::Container,
+                "Symbol" => TokenType::Symbol,
+                "Comment" => TokenType::Comment,
                 _ => return None,
             });
         }
         None
     }
 }
+
+/**
+A piece of the tokenized script.
+
+Usually a singular piece but may be a `Container`.
+*/
 #[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
 #[cfg_attr(feature = "compile", derive(Serialize, Deserialize))]
 pub struct Token {
@@ -307,12 +340,12 @@ pub struct Token {
     pub data: String,
 }
 impl Token {
-    pub fn from_value(v: &Value) -> Option<Self> {
+    pub(crate) fn from_value(v: &Value) -> Option<Self> {
         if let Value::Struct(hm) = v {
             return Some(Token {
-                ttype: TokenType::from_value(&hm.get("ttype")?.val)?,
-                pos: Pos::from_value(&hm.get("pos")?.val)?,
-                data: match &hm.get("data")?.val {
+                ttype: TokenType::from_value(&hm.get("ttype")?.clone_out().val)?,
+                pos: Pos::from_value(&hm.get("pos")?.clone_out().val)?,
+                data: match &hm.get("data")?.clone_out().val {
                     Value::String(s) => s.clone(),
                     _ => return None,
                 },
@@ -350,7 +383,7 @@ impl Iterator for Tokens {
         let mut tc: usize = 0;
         // Eat whitespace before token
         while tc < remainder.len() && remainder[tc].is_whitespace() {
-            match remainder[tc] {
+            match remainder.get(tc).unwrap() {
                 '\t' => self.pos.col += 4,
                 _ => self.pos.col += 1,
             }
@@ -408,7 +441,7 @@ impl Iterator for Tokens {
                     // Allow numbers, a single period, and a single 'e'/'E'
                     // TODO add support for hex and binary numbers
                     if !nc.is_numeric() {
-                        let token = self.remainder.get(0..tc+1)?.trim();
+                        let token = self.remainder.get(0..=tc)?.trim();
                         if token.matches('.').count() > 1
                             || token.to_lowercase().matches('e').count() > 1
                             || !token.to_lowercase()
@@ -431,7 +464,7 @@ impl Iterator for Tokens {
                 },
                 TokenType::String => {
                     // Remove escaped backslashes then remove escaped quotes to determine actual quotes
-                    let token = self.remainder.get(0..tc+1)?.trim();
+                    let token = self.remainder.get(0..=tc)?.trim();
                     let unescaped = token.replace(r"\\", "")
                         .replace(r#"\""#, "");
 
@@ -443,7 +476,7 @@ impl Iterator for Tokens {
                 TokenType::Container => {
                     // Check container matching
                     let mut containers = vec![];
-                    let token: Vec<char> = self.remainder.get(0..tc+1)?.trim()
+                    let token: Vec<char> = self.remainder.get(0..=tc)?.trim()
                         .chars().collect();
                     for c in &token {
                         match c {
@@ -477,7 +510,7 @@ impl Iterator for Tokens {
                     }
                 },
                 TokenType::Symbol => {
-                    let token = self.remainder.get(0..tc+1)?.trim();
+                    let token = self.remainder.get(0..=tc)?.trim();
                     match token {
                         "::" |
                         "**" |
@@ -491,7 +524,7 @@ impl Iterator for Tokens {
                         "&&" |
                         "||" |
                         ".." |
-                        "..." |
+                        "..=" |
                         "->" |
                         "<-" |
                         "=>" |
